//
// JTypist v3.1 - Java typing tutor program for UNIX systems
// Copyright (C) 2000  Simon Baldwin (simonb@sco.com)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
//
// KeyIcon class - implements a graphic of a key
//
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Frame;
import java.awt.BorderLayout;

/**
 * Implements a graphic of a key.
 */
public class KeyIcon extends KBIcon {

	// the legend positioning on the icon - may be upper left or central
	private boolean		position_upperleft;//	= false;


	/**
	 * Create a key icon with the specified legend, unit size,
	 * width, and height factor (in %).  A null legend
	 * creates a 'blank'.
	 */
	public KeyIcon( String legend, int unit_size, int width_factor,
			int height_factor ) {
		super( legend, unit_size, width_factor, height_factor );

		// now use our set_legend to get the special effects
		set_legend( legend );
	}

	/**
	 * Create a key icon with the specified legend, and width factor,
	 * and default unit size.
	 */
	public KeyIcon( String legend, int width_factor ) {
		this( legend, DEFAULT_SIZE, width_factor, 100 );
	}

	/**
	 * Paint the complete graphic.
	 */
	public synchronized void paint( Graphics g ) {
		Dimension d = getSize();

		// see if the legend signifies something other than a
		// blank keyicon
		if ( legend != null ) {

			// create an initial filled box
			Color colour = ( highlight_colour != null ) ?
					highlight_colour : getForeground();
			g.setColor( colour );
			g.fillRect( 2, 2, d.width - 4, d.height - 4 );

			// if the legend is supposed to be visible then
			// draw it, else don't
			if ( show_legend ) {

				// set a text colour depending on the fore-
				// ground, to make sure the legend shows up
				if ( ! colour.equals( Color.black ))
					g.setColor( Color.black );
				else
					g.setColor( Color.white );

				// put the legend in the right place in the box;
				// this is either upper left, or central
				FontMetrics fm	= g.getFontMetrics();
				if ( position_upperleft )
					g.drawString( legend,
						3, fm.getAscent() + 3 );
				else
					g.drawString( legend,
					( d.width  - fm.stringWidth( legend )) / 2,
					( d.height + fm.getAscent() ) / 2 );
			}

			// end with a 3d box round the item
			g.setColor( getBackground() );
			g.draw3DRect( 0, 0, d.width - 1, d.height - 1, true );
		}
		else {
			// simply blank the panel to the background colour
			g.setColor( getBackground() );
			g.fillRect( 0, 0, d.width - 1, d.height - 1 );
		}
	}


	/**
	 * Set the legend on the keyicon to be a different string.  Single
	 * character legends are set to uppercase, and shifted to the upper
	 * left of the key icon graphic.
	 */
	public synchronized void set_legend( String legend ) {

		// see if the new legend is null
		if ( legend == null ) {

			// just save a null legend
			this.legend		= null;
			position_upperleft	= false;

			// indicate that the graphics need updating
			repaint();
		}

		// see if the legend has changed at all
		else if ( ! legend.equals( this.legend ) ) {

			// see if this legend represents a single alphabetical;
			// this represents some intelligent guesswork about
			// how the legend should appear, and seems to work
			// adequately
			if ( legend.length() == 1 &&
				Character.isLetter( legend.charAt( 0 ) )) {

				// save it, converting to uppercase and setting
				// the position accordingly
				this.legend		= legend.toUpperCase();
				position_upperleft	= true;
			}
			else {

				// save it as it stands; no special requirements
				this.legend		= legend;
				position_upperleft	= false;
			}

			// indicate that the graphics need updating
			repaint();
		}

	}


	/**
	 * Class test code.
	 */
	public static void main( String args[] ) {
		System.out.println( "*** CLASS TEST CODE ***" );

		Frame frame = new Frame();
		KeyIcon keyicon = new KeyIcon( "Enter", 175 );
		frame.setLayout( new BorderLayout() );
		frame.add( "Center", keyicon );
		frame.pack(); frame.setVisible( true );
		while ( true ) {
			keyicon.set_colour( Color.red );
			try { Thread.sleep( 1000 ); }
			catch( InterruptedException e ) {}
			keyicon.set_colour( Color.green );
			try { Thread.sleep( 1000 ); }
			catch( InterruptedException e ) {}
			keyicon.set_colour( Color.black );
			try { Thread.sleep( 1000 ); }
			catch( InterruptedException e ) {}
		}
	}
}
